from __future__ import annotations
from pathlib import Path
import json


def detect_lang(path: Path) -> list[str]:
    """return lowercase language name, or empty if undetermined.

    Reference:
    https://help.github.com/en/github/visualizing-repository-data-with-graphs/
        listing-the-packages-that-a-repository-depends-on#supported-package-ecosystems
    """

    cmfn = path / "codemeta.json"
    if cmfn.is_file():
        try:
            meta = json.loads(cmfn.read_text(errors="ignore"))
            if "programmingLanguage" in meta:
                return [L.lower() for L in meta["programmingLanguage"]]
        except json.decoder.JSONDecodeError:
            pass

    # fallback to heuristic
    if (path / "pyproject.toml").is_file() or (path / "pipfile.lock").is_file():
        return ["python"]
    if (path / "package-lock.json").is_file() or (path / "yarn.lock").is_file():
        return ["javascript"]
    if (path / "composer.lock").is_file():
        return ["php"]
    if (path / "Gemfile.lock").is_file():
        return ["ruby"]

    return []



#!/usr/bin/env python3
r"""
Recursively find files containing text.
This method is slower than grep or findstr, but is cross-platform and easier syntax.

## benchmarks:

time findtext xarray
18.6 sec

# note there are no "" on the command below.
# It's the equivalent of the defaults for the Python script.
time grep -r -l \
  --exclude-dir={\_site,\.git,\.eggs,build,dist,\.mypy_cache,.pytest_cache,*\.egg-info} \
  --include=*.{py,cfg,ini,txt,pdf,md,rst,tex,f,f90,for,f95,c,h,cpp,hpp,m} \
  xarray .
0.55 sec

---
time findtext xarray "*.py"
1.14 sec

time grep -r -l \
  --exclude-dir={\_site,\.git,\.eggs,build,dist,\.mypy_cache,.pytest_cache,*\.egg-info} \
  --include=*.py  xarray .
0.15 sec

"""

from pathlib import Path
from argparse import ArgumentParser

from .project import detect_lang


def find_project():
    p = ArgumentParser(description="searches for projects with codemeta.json of specified language")
    p.add_argument("codelang", help="code language to search for")
    p.add_argument("dir", help="root dir to search")
    p.add_argument("-v", "--verbose", action="store_true")
    P = p.parse_args()

    # %% preflight
    root = Path(P.dir).expanduser().resolve()
    if not root.is_dir():
        raise SystemExit(f"{root} is not a directory.")

    req_lang = P.codelang.lower()

    dirs = (d for d in root.iterdir() if d.is_dir())

    for d in dirs:
        if req_lang in detect_lang(d):
            print(d)



#!/usr/bin/env python
import subprocess
import pytest
import pyfindfiles.text as pf
import os
from datetime import datetime
import sys

"""
TEST_STRING_TO_MATCH
"""


@pytest.mark.skipif(bool(os.environ.get("CI")), reason="CI does not like recursive search")
def test_script(tmp_path):
    (tmp_path / "foo.py").write_text("import datetime")
    ret = subprocess.check_output(
        [sys.executable, "-m", "pyfindfiles.text", "import", "*.py", str(tmp_path)],
        text=True,
    )

    assert isinstance(ret, str)

    assert len(ret) > 0


def test_glob(tmp_path):
    fn = tmp_path / "foo.py"
    fn.write_text("MATCH_ME_NOW_PLEASE")

    files = pf.findtext(tmp_path, "MATCH_ME_NOW", globext="*.py")

    for file, _ in files:
        assert file.samefile(fn)


def test_age(tmp_path):
    fn = tmp_path / "foo.py"
    fn.write_text("MATCH_ME_NOW_PLEASE")

    files = pf.findtext(tmp_path, "MATCH_ME_NOW", globext="*.py", age=[datetime(2024, 1, 1)])

    for file, _ in files:
        assert file.samefile(fn)



#!/usr/bin/env python
import pytest
from pathlib import Path
import subprocess
import sys

import pyfindfiles.video as fv

if __file__ is None:
    R = None
else:
    R = Path(__file__).parent


@pytest.mark.skipif(R is None, reason="__file__ missing")
def test_findvid_serial():
    files = fv.findvid(R, ".avi")

    flist = list(files)
    print(flist)
    assert len(flist) == 2


@pytest.mark.skipif(R is None, reason="__file__ missing")
@pytest.mark.skipif(sys.platform != "linux", reason="Linux only test")
def test_findvid_gnu():
    files = list(fv.findvid_gnu(R, ".avi"))

    assert len(files) == 2


@pytest.mark.skipif(R is None, reason="__file__ missing")
def test_script():
    files = subprocess.check_output(
        [sys.executable, "-m", "pyfindfiles.video", str(R)], text=True
    ).strip()
    flist = files.split("\n")
    print(flist)
    assert len(flist) == 2



import pyfindfiles.project as pf


def test_find_project(tmp_path):
    r = tmp_path
    (r / "pyproject.toml").touch()

    langs = pf.detect_lang(r)

    assert len(langs) == 1
    assert "python" in langs



from __future__ import annotations
import os
from pathlib import Path
import typing as T
from datetime import datetime
from argparse import ArgumentParser
import dateutil.parser
import shutil
import subprocess
import functools

from . import MAGENTA, BLACK

EXCLUDEDIR = ["_site", "_deps", ".git", ".eggs", "build", "dist", ".mypy_cache", ".pytest_cache"]

TXTEXT = [
    "*.py",
    "*.cfg",
    "*.ini",
    "*.txt",
    "*.md",
    "*.rst",
    "*.tex",
    "*.build",
    "*.cmake",
    "*.f",
    "*.f90",
    "*.for",
    "*.f95",
    "*.c",
    "*.h",
    "*.cpp",
    "*.cxx",
    "*.cc",
    "*.hpp",
    "*.m",
]

BINEXT = ["*.pdf"]

MAXSIZE = 10e6  # arbitrary, bytes


@functools.cache
def is_git_submodule(p: Path) -> bool:

    if not p.is_dir():
        raise NotADirectoryError("is_git_submodule() expects a directory")

    ret = subprocess.run(
        ["git", "-C", str(p), "rev-parse", "--show-superproject-working-tree"],
        stdout=subprocess.PIPE,
        stderr=subprocess.DEVNULL,
        text=True,
    )

    return ret.returncode == 0 and len(ret.stdout) > 0


def findtext(
    root: Path,
    txt: str,
    *,
    globext: list[str],
    exclude: list[str] | None = None,
    exclude_submod: bool = True,
    age: list[datetime] | None = None,
) -> T.Iterator[tuple[Path, dict[int, str]]]:
    """
    multiple extensions with braces like Linux does not work in .rglob()

    Parameters
    ----------
    root : pathlib.Path
        root directory to search
    txt : str
        text to search for
    globext : list[str]
        list of file extensions to search
    exclude : list[str], optional
        list of substring to exclude in path segment, by default None
    exclude_submod : bool, optional
        exclude Git submodules, by default True
    age : list[datetime], optional
        newer than date or between dates, by default None
    """

    root = Path(root).expanduser()
    if not root.is_dir():
        raise NotADirectoryError(root)

    if isinstance(globext, (str, Path)):
        globext = [str(globext)]

    exc = set(exclude) if exclude else None

    for ext in globext:
        for fn in root.rglob(ext):

            if not fn.is_file():
                continue

            finf = fn.stat()

            if finf.st_size > MAXSIZE:
                continue

            p = fn.resolve()

            # exclude substring in path segment
            if exc and exc.intersection(set(str(p).split(os.sep))):
                continue

            # exclude Git submodules
            if exclude_submod and is_git_submodule(p.parent):
                continue

            # newer than date or between dates
            if age is not None:
                mt = datetime.fromtimestamp(finf.st_mtime)
                if mt < age[0]:
                    continue
                if len(age) == 2 and mt > age[1]:
                    continue

            try:
                with p.open("r", encoding="utf8", errors="ignore") as f:
                    matches = {i: str(line) for i, line in enumerate(f) if txt in line}
            except PermissionError:
                continue

            if not matches:
                continue

            yield p, matches


def cli():
    p = ArgumentParser(description="searches for TEXT under DIR and echos back filenames")
    p.add_argument("txt", help="text to search for")  # required
    p.add_argument("globext", help="filename glob", nargs="?", default=TXTEXT)
    p.add_argument("dir", help="root dir to search", nargs="?", default=".")
    p.add_argument("-t", "--time", help="newer than date or between dates", nargs="+")
    p.add_argument("-c", "--run", help="command to run on files e.g. notepad++")
    p.add_argument("-e", "--exclude", help="exclude files/dirs", nargs="+", default=EXCLUDEDIR)
    p.add_argument("-x", "--exclude_submod", action="store_true", default=True)
    p.add_argument("-v", "--verbose", action="store_true")
    P = p.parse_args()

    # %% preflight
    root = Path(P.dir).expanduser().resolve()
    if not root.is_dir():
        raise SystemExit(f"{root} is not a directory.")

    if P.run:
        exe = shutil.which(P.run)  # necessary for some Windows program e.g. VScode
        if not exe:
            raise SystemExit(f"could not find {exe}")

    time = None
    if P.time:
        time = [dateutil.parser.parse(t) for t in P.time]
    # %% main
    for fn, matches in findtext(
        P.dir,
        P.txt,
        globext=P.globext,
        exclude=P.exclude,
        exclude_submod=P.exclude_submod,
        age=time,
    ):
        if P.verbose:
            print(MAGENTA + str(fn) + BLACK)
            for k, v in matches.items():
                print(k, ":", v)
        else:
            print(fn)

        if P.run:
            subprocess.run([exe, str(fn)])


if __name__ == "__main__":
    cli()



from __future__ import annotations
import typing
from pathlib import Path
import logging
import asyncio
import itertools


async def findvid_win_run(path: Path, exts: list[str]) -> typing.Iterator[Path]:
    futures = [findvid_win(path, ext) for ext in exts]
    return itertools.chain.from_iterable(filter(None, await asyncio.gather(*futures)))


async def findvid_win(path: Path, ext: str) -> list[Path]:
    """
    asynchronously find files with extension

    Parameters
    ----------

    path : pathlib.Path
        root directory to recursively search under
    ext : str
        file extension to look for

    Yields
    -------

    video: pathlib.Path
        path to video file
    """

    path = Path(path).expanduser()
    cmd = ["dir", "/s", "*" + ext]
    logging.debug(" ".join(cmd))
    # this has to be _shell due to that "dir" is part of Windows shell itself; _exec won't work.
    proc = await asyncio.create_subprocess_shell(
        " ".join(cmd),
        cwd=path,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.DEVNULL,
    )
    stdout, _ = await proc.communicate()

    flist = []
    for r in stdout.decode("utf8").split("\n"):
        if not r:
            continue

        el = r.split()
        if not el:
            continue

        if el[0].startswith("Directory"):
            d = Path(" ".join(el[2:]))
            continue

        if el[-1].endswith(ext):
            flist.append(d / el[-1])

    return flist



from __future__ import annotations
import typing as T
from pathlib import Path
import logging
import sys
import subprocess
import shutil
from argparse import ArgumentParser

VIDEXT = [".avi", ".mov", ".mp4", ".mpg", ".mpeg", ".webm", ".ogv", ".mkv", ".wmv"]


def findvid(path: Path, ext: list[str]) -> T.Iterator[Path]:
    """
    recursive file search in Pure Python.
    about 10 times slower than Linux find, but platform-independent.
    """

    path = Path(path).expanduser()

    for e in ext:
        for file in path.glob("**/*" + e):
            yield file


def findvid_gnu(path: Path, exts: list[str]) -> T.Iterator[str]:
    """
    recursive file search using GNU find
    """
    path = Path(path).expanduser()
    if isinstance(exts, str):
        exts = [exts]

    find = shutil.which("find")
    if not find:
        raise FileNotFoundError('could not find "find"')

    cmd = [find, str(path), "-type", "f"]

    if sys.platform != "darwin":
        cmd += ["-regextype", "posix-egrep"]

    cmd += ["-iregex", r".*(" + r"|".join(exts) + r")$"]

    logging.debug(" ".join(cmd))

    stdout = subprocess.check_output(cmd, text=True).strip()

    for file in stdout.split("\n"):
        yield file


if __name__ == "__main__":
    p = ArgumentParser()
    p.add_argument("path", help="root path to start recursive search")
    p.add_argument("-v", "--verbose", action="store_true")
    p.add_argument("-ext", help="video extension to search for", nargs="+", default=VIDEXT)
    P = p.parse_args()

    root = Path(P.path).expanduser().resolve()
    if not root.is_dir():
        raise NotADirectoryError(root)

    if P.verbose:
        logging.basicConfig(level=logging.DEBUG)

    if sys.platform == "linux":
        videos = findvid_gnu(root, P.ext)
    else:
        videos = findvid(root, P.ext)

    for video in videos:
        print(video)
